@@ -566,6 +566,7 @@ init_client_options(struct client_options *self,
   object_queue_init(&self->actions);
   
   self->resources = make_resource_list();
+  gc_global(&self->resources->super);
 }
 
 static const struct argp_option
@@ -1017,6 +1018,7 @@ make_client_session(struct client_options *options)
   debug("lsh.c: Setting up stdout\n");
 
   if (options->stdout_file)
+    /* FIXME: Use O_TRUNC too? */
     out = open(options->stdout_file, O_WRONLY | O_CREAT, 0666);
   else
     {
@@ -1033,6 +1035,7 @@ make_client_session(struct client_options *options)
   debug("lsh.c: Setting up stderr\n");
   
   if (options->stderr_file)
+    /* FIXME: Use O_TRUNC too? */
     err = open(options->stderr_file, O_WRONLY | O_CREAT, 0666);
   else
     {
