@@ -21,24 +21,41 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+
 #include "client_userauth.h"
 
 #include "charset.h"
 #include "command.h"
 #include "format.h"
+#include "lsh_string.h"
 #include "parse.h"
 #include "publickey_crypto.h"
 #include "ssh.h"
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-
 /* Forward declaration */
 struct client_userauth; 
 
 #include "client_userauth.c.x"
 
+static struct lsh_string *
+format_userauth_none(struct lsh_string *name,
+                     int service)
+{
+  return ssh_format("%c%S%a%a",
+		    SSH_MSG_USERAUTH_REQUEST,
+		    name,
+		    service,
+		    ATOM_NONE);
+}
+
+/* The last parameters says whether or not to free the password. */
 struct lsh_string *
 format_userauth_password(struct lsh_string *name,
 			 int service,
@@ -56,8 +73,8 @@ format_userauth_password(struct lsh_string *name,
 
 static struct lsh_string *
 format_userauth_publickey_query(struct lsh_string *name,
-				UINT32 service,
-				UINT32 keytype,
+				uint32_t service,
+				uint32_t keytype,
 				struct lsh_string *public)
 {
   return ssh_format("%c%S%a%a%c%a%S",
@@ -72,8 +89,8 @@ format_userauth_publickey_query(struct lsh_string *name,
 
 static struct lsh_string *
 format_userauth_publickey(struct lsh_string *name,
-			  UINT32 service,
-			  UINT32 keytype,
+			  uint32_t service,
+			  uint32_t keytype,
 			  struct lsh_string *public)
 {
   return ssh_format("%c%S%a%a%c%a%S",
@@ -86,6 +103,54 @@ format_userauth_publickey(struct lsh_string *name,
 		    public);
 }
 
+static struct lsh_string *
+format_userauth_kbdinteract(struct lsh_string *name, uint32_t service)
+{
+  struct lsh_string *s;
+  s = ssh_format("%c%S%a%a%i%i",
+		 SSH_MSG_USERAUTH_REQUEST,
+		 name, service,
+		 /* Empty language tag and submethods */
+		 ATOM_KEYBOARD_INTERACTIVE, 0, 0);
+  debug("format_userauth_kbdinteract %xS\n", s);
+  return s;
+}
+
+static struct lsh_string *
+format_userauth_info_response(struct interact_dialog *dialog)
+{
+  uint32_t length;
+  unsigned i;
+  struct lsh_string *msg;
+  uint32_t p;
+
+  /* We need to format a message containing a variable number of
+     strings. */
+
+  /* First convert to utf8 */
+  for (i = 0; i < dialog->nprompt; i++)
+    dialog->response[i] = local_to_utf8(dialog->response[i], 1);
+  
+  for (i = length = 0; i < dialog->nprompt; i++)
+    length += lsh_string_length(dialog->response[i]);
+
+  msg = ssh_format("%c%i%lr", SSH_MSG_USERAUTH_INFO_RESPONSE, dialog->nprompt,
+		   length + 4 * dialog->nprompt, &p);
+
+  for (i = 0; i < dialog->nprompt; i++)
+    {
+      struct lsh_string *r = dialog->response[i];
+      uint32_t rlength = lsh_string_length(r);
+      lsh_string_write_uint32(msg, p, rlength);
+      p += 4;
+      lsh_string_write(msg, p, rlength, lsh_string_data(r));
+      p += rlength;
+    }
+  assert (p == lsh_string_length(msg));
+  
+  return msg;
+}
+
 /* Called when we receive a USERAUTH_FAILURE message. It will
  * typically either try again, or raise EXC_USERAUTH. */
 
@@ -126,6 +191,7 @@ format_userauth_publickey(struct lsh_string *name,
      (name client_userauth)
      (super command)
      (vars
+       ; The name should already be converted to utf8
        (username string)            ; Remote user name to authenticate as.
        (service_name . int)         ; Service we want to access.
        (methods object object_list) ; Authentication methods, in order.
@@ -192,7 +258,7 @@ do_userauth_success(struct packet_handler *c,
 
   trace("client_userauth.c: do_userauth_success\n");
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_USERAUTH_SUCCESS)
@@ -200,8 +266,6 @@ do_userauth_success(struct packet_handler *c,
     {
       unsigned i;
       
-      lsh_string_free(packet);
-
       verbose("User authentication successful.\n");
 
       for (i = SSH_FIRST_USERAUTH_GENERIC; i < SSH_FIRST_CONNECTION_GENERIC; i++) 
@@ -210,10 +274,7 @@ do_userauth_success(struct packet_handler *c,
       COMMAND_RETURN(self->c, connection);
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid USERAUTH_SUCCESS message");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid USERAUTH_SUCCESS message");
 }
 
 static struct packet_handler *
@@ -250,6 +311,14 @@ userauth_method_is_useful(struct client_userauth *userauth,
 
   unsigned i;
 
+#if 0
+  debug("userauth_method_is_useful, advertised:\n");
+  for(i = 0; i < LIST_LENGTH(advertised); i++)
+    debug("  %a\n", LIST(advertised)[i]);
+
+  debug("  method: type = %a, class = %t\n", method->type, method);
+#endif
+  
   for(i = 0; i < LIST_LENGTH(advertised); i++)
     if (LIST(advertised)[i] == method->type)
       return 1;
@@ -272,7 +341,7 @@ do_userauth_failure(struct packet_handler *c,
   trace("client_userauth.c: do_userauth_failure\n");
   assert(self->state->current < self->state->next);
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_USERAUTH_FAILURE)
@@ -280,8 +349,6 @@ do_userauth_failure(struct packet_handler *c,
       && parse_boolean(&buffer, &partial_success)
       && parse_eod(&buffer))
     {
-      lsh_string_free(packet);
-      
       if (partial_success)
 	/* Doesn't help us */
 	werror("Received SSH_MSH_USERAUTH_FAILURE "
@@ -297,10 +364,7 @@ do_userauth_failure(struct packet_handler *c,
 							methods, self->state->current));
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid USERAUTH_FAILURE message.");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid USERAUTH_FAILURE message.");
   
   KILL(methods);	
 }
@@ -326,13 +390,13 @@ do_userauth_banner(struct packet_handler *self UNUSED,
   struct simple_buffer buffer;
 
   unsigned msg_number;
-  UINT32 length;
-  const UINT8 *msg;
+  uint32_t length;
+  const uint8_t *msg;
 
-  UINT32 language_length;
-  const UINT8 *language;
+  uint32_t language_length;
+  const uint8_t *language;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_USERAUTH_BANNER)
@@ -345,8 +409,6 @@ do_userauth_banner(struct packet_handler *self UNUSED,
     }
   else
     PROTOCOL_ERROR(connection->e, "Invalid USERAUTH_SUCCESS message");
-
-  lsh_string_free(packet);
 }
 
 static struct packet_handler userauth_banner_handler =
@@ -429,6 +491,8 @@ do_exc_userauth_disconnect(struct exception_handler *s,
       EXCEPTION_RAISE(self->connection->e,
 		      make_protocol_exception(SSH_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE,
 					      NULL));
+      /* Also raise the exception with the parent exception
+	 handler. */
     }
 
   EXCEPTION_RAISE(s->parent, e);
@@ -496,14 +560,78 @@ make_client_userauth(struct lsh_string *username,
   NEW(client_userauth, self);
 
   self->super.call = do_client_userauth;
-  self->username = username;
+  self->username = local_to_utf8(username, 1);
   self->service_name = service_name;
   self->methods = methods;
 
   return &self->super;
 }
 
-   
+
+/* None authentication */
+/* GABA:
+   (class
+     (name client_none_state)
+     (super client_userauth_failure)
+     (vars
+       (e object exception_handler)))
+*/
+
+static void
+do_none_failure(struct client_userauth_failure *s, int again)
+{
+  CAST(client_none_state, self, s);
+  
+  static const struct exception need_auth =
+    STATIC_EXCEPTION(EXC_USERAUTH, "Need real authentication.");
+
+  trace("client_userauth.c: do_none_failure\n");
+
+  if (again)
+    werror("Odd. Server says we should try the `none'authentication method again.\n");
+
+  EXCEPTION_RAISE(self->e, &need_auth);
+}
+
+static struct client_userauth_failure *
+make_client_none_state(struct exception_handler *e)
+{
+  NEW(client_none_state, self);
+
+  trace("client_userauth.c: make_client_none_state\n");
+
+  self->super.failure = do_none_failure;
+  self->e = e;
+
+  return &self->super;
+}
+
+static struct client_userauth_failure *
+do_none_login(struct client_userauth_method *s UNUSED,
+              struct client_userauth *userauth,
+              struct ssh_connection *connection,
+              struct exception_handler *e)
+{
+  verbose("Requesting authentication using the `none' method.\n");
+  
+  connection_send(connection,
+		  format_userauth_none(userauth->username,
+				       userauth->service_name));
+  
+  return make_client_none_state(e);
+}
+
+static struct client_userauth_method
+client_userauth_none =
+  { STATIC_HEADER, ATOM_NONE, do_none_login };
+
+struct client_userauth_method *
+make_client_none_auth(void)
+{
+  return &client_userauth_none;
+}
+
+
 /* Password authentication */
 
 #define MAX_PASSWD 100
@@ -527,12 +655,12 @@ send_password(struct client_password_state *state)
   struct lsh_string *passwd
     = INTERACT_READ_PASSWORD(state->tty, MAX_PASSWD,
 			     ssh_format("Password for %lS: ",
-					state->userauth->username), 1);
+					state->userauth->username));
 
   if (passwd)
     {
       /* Password empty? */
-      if (!passwd->length)
+      if (!lsh_string_length(passwd))
 	{
 	  /* NOTE: At least on some systems, the getpass function
 	   * sets the tty to raw mode, disabling ^C, ^D and the like.
@@ -543,11 +671,13 @@ send_password(struct client_password_state *state)
 	  state->tried_empty_passwd++;
 	}
 
-      C_WRITE(state->connection,
-	      format_userauth_password(local_to_utf8(state->userauth->username, 0),
-				       state->userauth->service_name,
-				       local_to_utf8(passwd, 1),
-				       1));
+      verbose("Requesting authentication using the `password' method.\n");
+      
+      connection_send(state->connection,
+		      format_userauth_password(state->userauth->username,
+					       state->userauth->service_name,
+					       local_to_utf8(passwd, 1),
+					       1));
     }
   else
     {
@@ -611,21 +741,23 @@ make_client_password_state(struct client_userauth *userauth,
   return self;
 }
 
+/* Used for both "password" and "keyboard-interactive" */
 /* GABA:
    (class
-     (name client_password_method)
+     (name client_userauth_interactive_method)
      (super client_userauth_method)
      (vars
        (tty object interact)))
 */
 
+
 static struct client_userauth_failure *
 do_password_login(struct client_userauth_method *s,
 		  struct client_userauth *userauth,
 		  struct ssh_connection *connection,
 		  struct exception_handler *e)
 {
-  CAST(client_password_method, self, s);
+  CAST(client_userauth_interactive_method, self, s);
   
   struct client_password_state *state
     = make_client_password_state(userauth, self->tty,
@@ -641,7 +773,8 @@ do_password_login(struct client_userauth_method *s,
 struct client_userauth_method *
 make_client_password_auth(struct interact *tty)
 {
-  NEW(client_password_method, self);
+  NEW(client_userauth_interactive_method, self);
+
   self->super.type = ATOM_PASSWORD;
   self->super.login = do_password_login;
   self->tty = tty;
@@ -683,10 +816,10 @@ make_client_password_auth(struct interact *tty)
        (keys object object_list)
        ; Number of keys for which we have received either
        ; a USERAUTH_FAILURE or USERAUTH_PK_OK message.
-       (done . UINT32)
+       (done . uint32_t)
        ; Number of keys for which we have computed and sent a signature,
        ; and not yet received any failure.
-       (pending . UINT32)
+       (pending . uint32_t)
        (e object exception_handler)))
 */
 
@@ -706,7 +839,7 @@ client_publickey_next(struct client_publickey_state *state)
 	 * SSH_MSG_USERAUTH_PK_OK */
 	return;
 
-      /* We have got a response on the final query request. */
+      /* We have got a resposne on the final query request. */
       state->connection->dispatch[SSH_MSG_USERAUTH_PK_OK]
 	= &connection_fail_handler;
     }
@@ -762,7 +895,7 @@ make_client_publickey_state(struct client_userauth *userauth,
 static void 
 do_userauth_pk_ok(struct packet_handler *s,
 		  struct ssh_connection *connection,
-		  struct lsh_string *packet UNUSED)
+		  struct lsh_string *packet)
 {
   CAST(userauth_pk_ok_handler, self, s);
 
@@ -770,10 +903,10 @@ do_userauth_pk_ok(struct packet_handler *s,
 
   unsigned msg_number;
   int algorithm;
-  UINT32 keyblob_length;
-  const UINT8 *keyblob;
+  uint32_t keyblob_length;
+  const uint8_t *keyblob;
 
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_USERAUTH_PK_OK)
@@ -789,38 +922,31 @@ do_userauth_pk_ok(struct packet_handler *s,
 	{
 	  struct lsh_string *request;
 	  struct lsh_string *signed_data;
-    
-#if DATAFELLOWS_WORKAROUNDS
-	  if (connection->peer_flags & PEER_USERAUTH_REQUEST_KLUDGE)
-	    request = format_userauth_publickey(local_to_utf8(self->state->userauth->username, 0),
-						ATOM_SSH_USERAUTH,
-						key->type,
-						key->public);
-	  else
-#endif /* DATAFELLOWS_WORKAROUNDS */ 
-	    request = format_userauth_publickey(local_to_utf8(self->state->userauth->username, 0),
-						self->state->userauth->service_name,
-						key->type,
-						key->public);
+
+	  verbose("Sending `publickey' signature.\n");
+  
+	  request = format_userauth_publickey(self->state->userauth->username,
+					      self->state->userauth->service_name,
+					      key->type,
+					      key->public);
+
 	  signed_data = ssh_format("%S%lS", connection->session_id, request);
 	  request = ssh_format("%flS%fS", 
 			       request, 
-			       SIGN(key->private, key->type, signed_data->length, signed_data->data));
+			       SIGN(key->private, key->type,
+				    lsh_string_length(signed_data),
+				    lsh_string_data(signed_data)));
 	  lsh_string_free(signed_data);
-	  C_WRITE(connection, request);
+	  connection_send(connection, request);
 	  self->state->pending++;
 	}
       else
 	werror("client_userauth.c: Unexpected key in USERAUTH_PK_OK message.\n");
 
-      lsh_string_free(packet);
       client_publickey_next(self->state);
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(self->state->e, "Invalid USERAUTH_PK_OK message");
-    }
+    PROTOCOL_ERROR(self->state->e, "Invalid USERAUTH_PK_OK message");
 }
 
 static struct packet_handler *
@@ -857,10 +983,13 @@ do_publickey_login(struct client_userauth_method *s,
       {
 	CAST(keypair, key, LIST(self->keys)[i]);
 
-	C_WRITE(connection, 
-		format_userauth_publickey_query(local_to_utf8(userauth->username, 0),
-						userauth->service_name,
-						key->type, key->public));
+	verbose("Requesting authentication using the `publickey' method.\n");
+	
+	connection_send(connection, 
+			format_userauth_publickey_query
+			  (userauth->username,
+			   userauth->service_name,
+			   key->type, key->public));
       }
     connection->dispatch[SSH_MSG_USERAUTH_PK_OK] = make_pk_ok_handler(state);
     return &state->super;
@@ -872,8 +1001,234 @@ make_client_publickey_auth(struct object_list *keys)
 {
   NEW(client_publickey_method, self);
 
+  self->super.type = ATOM_PUBLICKEY;
   self->super.login = do_publickey_login;
   self->keys = keys;
   
   return &self->super;
 }
+
+
+/* "keyboard-interact" authentication
+ *
+ * Problems with the spec (draft-ietf-secsh-auth-kbdinteract-06.txt):
+ *
+ *  * The deprecated language tags.
+ *
+ *  * Use of the type int, which is undefined (should be uint32_t).
+ *
+ *  * The supplied name. What is it supposed to mean if that differs
+ *    from the name in the USERAUTH_REQUEST?
+ *
+ * *  The submethods list use utf8. Should be ordinary ssh-names, in
+ *    ascii and with the @domain convention for non-standard methods.
+ */
+
+/* GABA:
+   (class
+     (name client_kbdinteract_state)
+     (super client_userauth_failure)
+     (vars
+       (tty object interact)
+       (connection object ssh_connection)
+       (e object exception_handler)))
+*/
+
+/* GABA:
+   (class
+     (name userauth_info_request_handler)
+     (super packet_handler)
+     (vars 
+       (state object client_kbdinteract_state)))
+*/
+
+#define KBDINTERACT_MAX_PROMPTS 17
+#define KBDINTERACT_MAX_LENGTH 200
+
+/* FIXME: Doesn't get control character filtering right. */
+
+static void 
+do_userauth_info_request(struct packet_handler *s,
+			 struct ssh_connection *connection,
+			 struct lsh_string *packet)
+{
+  CAST(userauth_info_request_handler, self, s);
+
+  struct simple_buffer buffer;
+
+  unsigned msg_number;
+  const uint8_t *name;
+  uint32_t name_length;
+
+  const uint8_t *instruction;
+  uint32_t instruction_length;
+  /* Deprecated and ignored */
+  const uint8_t *language;
+  uint32_t language_length;
+
+  /* Typed as "int" in the spec. Hope that means uint32_t? */  
+  uint32_t nprompt; 
+  
+  simple_buffer_init(&buffer, STRING_LD(packet));
+  if (parse_uint8(&buffer, &msg_number)
+      && (msg_number == SSH_MSG_USERAUTH_INFO_REQUEST)
+      && parse_string(&buffer, &name_length, &name)
+      && parse_string(&buffer, &instruction_length, &instruction)
+      && parse_string(&buffer, &language_length, &language)
+      && parse_uint32(&buffer, &nprompt))
+    {
+      struct interact_dialog *dialog;
+      unsigned i;
+      
+      if (nprompt > KBDINTERACT_MAX_PROMPTS
+	  || name_length > KBDINTERACT_MAX_LENGTH
+	  || instruction_length > 10*KBDINTERACT_MAX_LENGTH)
+	{
+	  static const struct exception bad_info_request =
+	    STATIC_EXCEPTION(EXC_USERAUTH, "Too large USERAUTH_INFO_RQUEST");
+	beyond_limit:
+
+	  EXCEPTION_RAISE(self->state->e, &bad_info_request);
+	  return;
+	}
+
+      dialog = make_interact_dialog(nprompt);
+
+      for (i = 0; i < nprompt; i++)
+	{
+	  const uint8_t *prompt;
+	  uint32_t prompt_length;
+	  struct lsh_string *s;
+
+	  if (! (parse_string(&buffer, &prompt_length, &prompt)
+		 && parse_boolean(&buffer, &dialog->echo[i])))
+	    {
+	      KILL(dialog);
+	      goto error;
+	    }
+
+	  if (prompt_length > KBDINTERACT_MAX_LENGTH)
+	    {
+	      KILL(dialog);
+	      goto beyond_limit;
+	    }
+	  s = low_utf8_to_local(prompt_length, prompt,
+				utf8_replace | utf8_paranoid);
+	  if (!s)
+	    goto error;
+	  
+	  dialog->prompt[i] = s;
+	}
+      
+      dialog->instruction
+	= low_utf8_to_local(instruction_length, instruction,
+			    utf8_replace | utf8_paranoid);
+      
+      if (!dialog->instruction)
+	goto error;
+
+      if (name_length > 0)
+	{
+	  /* Prepend to instruction */
+	  struct lsh_string *s;
+      
+	  s = low_utf8_to_local(name_length, name,
+				utf8_replace | utf8_paranoid);
+	  if (!s)
+	    goto error;
+
+	  dialog->instruction = ssh_format("%lfS\n\n%lfS\n",
+					   s, dialog->instruction);
+	}
+      else
+	dialog->instruction = ssh_format("%lfS\n", dialog->instruction);
+
+      if (!INTERACT_DIALOG(self->state->tty, dialog))
+	{
+	  static const struct exception bad_info_request =
+	    STATIC_EXCEPTION(EXC_USERAUTH, "No user response");
+
+	  EXCEPTION_RAISE(self->state->e, &bad_info_request);
+	  return;
+	}
+
+      connection_send(connection, format_userauth_info_response(dialog));
+      KILL(dialog);
+      return;
+    }
+  else
+    {
+    error:
+      PROTOCOL_ERROR(self->state->e, "Invalid USERAUTH_INFO_REQUEST message");
+    }
+}
+
+static struct packet_handler *
+make_userauth_info_request_handler(struct client_kbdinteract_state *state)
+{
+  NEW(userauth_info_request_handler, self);
+  self->super.handler = do_userauth_info_request;
+  self->state = state;
+
+  return &self->super;
+}
+
+static void
+do_kbdinteract_failure(struct client_userauth_failure *s, int again UNUSED)
+{
+  CAST(client_kbdinteract_state, self, s);
+
+  /* FIXME: We never try again. How do we know if the user wants to
+     move on? */
+  static const struct exception kbdinteract_fail =
+    STATIC_EXCEPTION(EXC_USERAUTH, "keyboard-interact authentication failed.");
+
+  EXCEPTION_RAISE(self->e, &kbdinteract_fail);  
+}
+
+static struct client_kbdinteract_state *
+make_client_kbdinteract_state(struct interact *tty,
+			      struct ssh_connection *connection,
+			      struct exception_handler *e)
+{
+  NEW(client_kbdinteract_state, self);
+  self->super.failure = do_kbdinteract_failure;
+  self->tty = tty;
+  self->connection = connection;
+  self->e = e;
+
+  return self;
+}
+
+static struct client_userauth_failure *
+do_kbdinteract_login(struct client_userauth_method *s,
+		     struct client_userauth *userauth,
+		     struct ssh_connection *connection,
+		     struct exception_handler *e)
+{
+  CAST(client_userauth_interactive_method, self, s);
+  struct client_kbdinteract_state *state
+    = make_client_kbdinteract_state(self->tty, connection, e);
+
+  verbose("Requesting authentication using the `keyboard-interactive' method.\n");
+  
+  connection_send(connection,
+		  format_userauth_kbdinteract(userauth->username, userauth->service_name));
+
+  connection->dispatch[SSH_MSG_USERAUTH_INFO_REQUEST]
+    = make_userauth_info_request_handler(state);
+
+  return &state->super;
+}
+
+struct client_userauth_method *
+make_client_kbdinteract_auth(struct interact *tty)
+{
+  NEW(client_userauth_interactive_method, self);
+
+  self->super.type = ATOM_KEYBOARD_INTERACTIVE;
+  self->super.login = do_kbdinteract_login;
+  self->tty = tty;
+  
+  return &self->super;
+}
