@@ -21,23 +21,26 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+
 #include "connection.h"
 
+#include "command.h"
 #include "compress.h"
-#include "debug.h"
-#include "encrypt.h"
 #include "exception.h"
 #include "format.h"
 #include "io.h"
 #include "keyexchange.h"
-#include "pad.h"
+#include "lsh_string.h"
 #include "parse.h"
 #include "ssh.h"
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-
 #define GABA_DEFINE
 #include "connection.h.x"
 #undef GABA_DEFINE
@@ -52,11 +55,12 @@ static void
 connection_handle_packet(struct ssh_connection *closure,
 			 struct lsh_string *packet)
 {
-  UINT8 msg;
-
+  uint8_t msg;
+  uint32_t length = lsh_string_length(packet);
+  
   assert(!closure->paused);
   
-  if (!packet->length)
+  if (!length)
     {
       werror("connection.c: Received empty packet!\n");
       PROTOCOL_ERROR(closure->e, "Received empty packet.");
@@ -64,7 +68,7 @@ connection_handle_packet(struct ssh_connection *closure,
       return;
     }
 
-  if (packet->length > closure->rec_max_packet)
+  if (length > closure->rec_max_packet)
     {
       werror("connection.c: Packet too large!\n");
       PROTOCOL_ERROR(closure->e, "Packet too large");
@@ -72,7 +76,7 @@ connection_handle_packet(struct ssh_connection *closure,
       return;
     }
     
-  msg = packet->data[0];
+  msg = lsh_string_data(packet)[0];
 
   debug("handle_connection: Received packet of type %i (%z)\n",
 	msg, packet_types[msg]);
@@ -191,38 +195,38 @@ DEFINE_PACKET_HANDLER(, connection_fail_handler, connection, packet UNUSED)
 DEFINE_PACKET_HANDLER(, connection_unimplemented_handler, connection, packet)
 {
   werror("Received packet of unimplemented type %i.\n",
-	 packet->data[0]);
+	 lsh_string_data(packet)[0]);
 
-  C_WRITE(connection,
+  connection_send(connection,
 	  ssh_format("%c%i",
 		     SSH_MSG_UNIMPLEMENTED,
-		     packet->sequence_number));
+		     lsh_string_sequence_number(packet)));
 }
 
 DEFINE_PACKET_HANDLER(, connection_forward_handler, connection, packet)
 {
   assert(connection->chain);
   /* FIXME: Packets of certain types (IGNORE, DEBUG, DISCONNECT)
-   * could be sent with C_WRITE_NOW. */
-  C_WRITE(connection->chain, packet);
+   * could be sent with connection_send_kex. */
+  connection_send(connection->chain, packet);
 }
 
 DEFINE_PACKET_HANDLER(, connection_disconnect_handler, connection, packet)
 {
   struct simple_buffer buffer;
   unsigned msg_number;
-  UINT32 reason;
+  uint32_t reason;
 
-  UINT32 length;
-  const UINT8 *msg;
+  uint32_t length;
+  const uint8_t *msg;
 
-  const UINT8 *language;
-  UINT32 language_length;
+  const uint8_t *language;
+  uint32_t language_length;
   
   static const struct exception disconnect_exception =
     STATIC_EXCEPTION(EXC_FINISH_IO, "Received disconnect message.");
     
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_DISCONNECT)
@@ -274,38 +278,13 @@ do_exc_connection_handler(struct exception_handler *s,
         werror("Protocol error: %z\n", e->msg);
         
 	if (exc->reason)
-	  C_WRITE_NOW(self->connection,
-		      format_disconnect(exc->reason, exc->super.msg, ""));
+	  connection_send_kex(self->connection,
+			      format_disconnect(exc->reason, exc->super.msg, ""));
 	
 	EXCEPTION_RAISE(self->super.parent, &finish_read_exception);
       }
       break;
 
-    case EXC_PAUSE_CONNECTION:
-      assert(!self->connection->paused);
-      self->connection->paused = 1;
-      EXCEPTION_RAISE(self->super.parent,
-		      make_simple_exception(EXC_PAUSE_READ, "locking connection."));
-      break;
-
-    case EXC_PAUSE_START_CONNECTION:
-      /* NOTE: Raising EXC_PAUSE_START_READ will not by itself make
-       * the connection start processing pending packets. Not until
-       * the peer sends us more data.
-       *
-       * So any code that raises EXC_PAUSE_START_CONNECTION should
-       * also call connection_handle_pending at a safe place. We
-       * can't call it here, as we may be in the middle of the
-       * handling of a packet. Installing a callout would be best. */
-      
-      assert(self->connection->paused);
-      EXCEPTION_RAISE(self->super.parent,
-		      make_simple_exception(EXC_PAUSE_START_READ, "unlocking connection."));
-      
-      self->connection->paused = 0;
-
-      break;
-
     case EXC_FINISH_READ:
       /* FIXME: We could check if there are any channels still open,
        * and write a warning message if there are. */
@@ -332,6 +311,47 @@ make_exc_connection_handler(struct ssh_connection *connection,
   return &self->super;
 }
 
+/* About three max size packets */
+#define WRITE_BUFFER_MAX    100000
+/* Should be enough for a reasonable number of keyexchange packets or
+ * keepalive packets */
+#define WRITE_BUFFER_MARGIN  10000
+
+/* GABA:
+   (class
+     (name connection_flow_controlled)
+     (super flow_controlled)
+     (vars
+       (connection object ssh_connection)))
+*/
+
+static void
+do_connection_flow_controlled(struct flow_controlled *s,
+			      uint32_t written UNUSED)
+{
+  CAST(connection_flow_controlled, self, s);
+  struct ssh_connection *connection = self->connection;
+  uint32_t length = connection->socket->write_buffer->length;
+  
+  if (connection->hard_limit
+      && length + WRITE_BUFFER_MARGIN < connection->soft_limit)
+    {
+      trace("do_connection_flow_controlled: Resetting hard_limit\n");
+      connection->hard_limit = 0;
+      if (connection->wakeup)
+	COMMAND_RETURN(connection->wakeup, connection);
+    }
+}
+
+static struct flow_controlled *
+make_connection_flow_controlled(struct ssh_connection *connection)
+{
+  NEW(connection_flow_controlled, self);
+  self->super.report = do_connection_flow_controlled;
+  self->connection = connection;
+  return &self->super;
+}
+
 struct ssh_connection *
 make_ssh_connection(enum connection_flag flags,
 		    struct address_info *peer,
@@ -355,9 +375,8 @@ make_ssh_connection(enum connection_flag flags,
   connection->e = make_exc_connection_handler(connection, e, HANDLER_CONTEXT);
 
   connection->keyexchange_done = NULL;
+  connection->wakeup = NULL;
   
-  /* Initialize instance variables */
-
   connection->versions[CONNECTION_SERVER]
     = connection->versions[CONNECTION_CLIENT]
     = connection->session_id = NULL;
@@ -388,13 +407,15 @@ make_ssh_connection(enum connection_flag flags,
 
   connection->send_kex_only = 0;
   string_queue_init(&connection->send_queue);
+  connection->soft_limit = WRITE_BUFFER_MAX;
+  connection->hard_limit = 0;
   
   connection->kexinits[CONNECTION_CLIENT]
     = connection->kexinits[CONNECTION_SERVER] = NULL;
 
   connection->literal_kexinits[CONNECTION_CLIENT]
     = connection->literal_kexinits[CONNECTION_SERVER] = NULL;
-  
+
   for (i = 0; i < 0x100; i++)
     connection->dispatch[i] = &connection_unimplemented_handler;
 
@@ -440,22 +461,11 @@ make_ssh_connection(enum connection_flag flags,
 
 void
 connection_init_io(struct ssh_connection *connection,
-		   struct abstract_write *raw,
-		   struct randomness *r)
+		   struct lsh_fd *socket)
 {
   /* Initialize i/o hooks */
-  connection->raw = raw;
-  connection->write_packet =
-    make_packet_debug(
-      make_packet_deflate(
-	make_packet_pad(
-	  make_packet_encrypt(raw, connection), 
-	  connection,
-	  r),
-	connection),
-      (connection->debug_comment
-       ? ssh_format("%lz sent", connection->debug_comment)
-       : ssh_format("Sent")));
+  connection->socket = socket;
+  connection->socket->write_buffer->report = make_connection_flow_controlled(connection);
 }
 
 void
@@ -466,6 +476,14 @@ connection_after_keyexchange(struct ssh_connection *self,
   self->keyexchange_done = c;
 }
 
+void
+connection_wakeup(struct ssh_connection *self,
+		  struct command_continuation *c)
+{
+  assert(!self->wakeup);
+  self->wakeup = c;
+}
+
 /* GABA:
    (class
      (name connection_close_handler)
@@ -495,8 +513,33 @@ make_connection_close_handler(struct ssh_connection *c)
   return &closure->super;
 }
 
+/* Processes the packet at once, passing it on to the write buffer. */
+void
+connection_send_kex(struct ssh_connection *self,
+		    struct lsh_string *message)
+{
+  uint32_t length;
+  
+  A_WRITE(self->write_packet, message);
 
-/* Sending ordinary (non keyexchange) packets */
+  length = self->socket->write_buffer->length;
+  if (self->hard_limit)
+    {
+      if (length > self->hard_limit)
+	{
+	  static struct protocol_exception disconnect =
+	    STATIC_PROTOCOL_EXCEPTION(0, "Write buffer full, peer not responding.");
+	  EXCEPTION_RAISE(self->e, &disconnect.super);
+	}
+    }
+  else if (length > self->soft_limit)
+    {
+      self->hard_limit = length + WRITE_BUFFER_MARGIN;
+      trace("connection_send_kex: Setting hard limit to %i\n", self->hard_limit);
+    }
+}
+
+/* Sends one ordinary (non keyexchange) packet */
 void
 connection_send(struct ssh_connection *self,
 		struct lsh_string *message)
@@ -508,7 +551,7 @@ connection_send(struct ssh_connection *self,
   else
     {
       assert(string_queue_is_empty(&self->send_queue));
-      C_WRITE_NOW(self, message);
+      connection_send_kex(self, message);
     }
 }
 
@@ -526,7 +569,7 @@ connection_send_kex_end(struct ssh_connection *self)
   assert(self->send_kex_only);
 
   while (!string_queue_is_empty(&self->send_queue))
-    C_WRITE_NOW(self, string_queue_remove_head(&self->send_queue));
+    connection_send_kex(self, string_queue_remove_head(&self->send_queue));
 
   self->send_kex_only = 0;
 
@@ -537,24 +580,42 @@ connection_send_kex_end(struct ssh_connection *self)
   
       COMMAND_RETURN(c, self);
     }
+  if (self->wakeup)
+    COMMAND_RETURN(self->wakeup, self);
 }
 
 /* Serialization. */
 
-void connection_lock(struct ssh_connection *self)
+void
+connection_lock(struct ssh_connection *self)
 {
   const struct exception pause
-    = STATIC_EXCEPTION(EXC_PAUSE_CONNECTION, "locking connection.");
-    
+    = STATIC_EXCEPTION(EXC_PAUSE_READ, "locking connection.");
+  
+  assert(!self->paused);
+  self->paused = 1;
   EXCEPTION_RAISE(self->e, &pause);
 }
 
-void connection_unlock(struct ssh_connection *self)
+void
+connection_unlock(struct ssh_connection *self)
 {
   const struct exception unpause
-    = STATIC_EXCEPTION(EXC_PAUSE_START_CONNECTION, "unlocking connection.");
-    
+    = STATIC_EXCEPTION(EXC_PAUSE_START_READ, "unlocking connection.");
+
+  /* NOTE: Raising EXC_PAUSE_START_READ will not by itself make
+   * the connection start processing pending packets. Not until
+   * the peer sends us more data.
+   *
+   * So any code that calls connection_unlock should also call
+   * connection_handle_pending at a safe place. We can't call it here,
+   * as we may be in the middle of the handling of a packet.
+   * Installing a callout would be best. */
+  
+  assert(self->paused);
   EXCEPTION_RAISE(self->e, &unpause);
+      
+  self->paused = 0;
 }
 
 
