@@ -1,6 +1,5 @@
 /* gateway_commands.h
  *
- * $Id: gateway_commands.h,v 1.2 2001/07/06 20:06:42 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -25,18 +24,15 @@
 #ifndef GATEWAY_COMMANDS_H_INCLUDED
 #define GATEWAY_COMMANDS_H_INCLUDED
 
-#include "command.h"
+#include "io.h"
 
-extern struct command gateway_init;
-#define GATEWAY_INIT (&gateway_init.super)
+extern struct command_2 gateway_init;
+#define GATEWAY_INIT (&gateway_init.super.super)
 
 extern struct command_2 gateway_accept;
 #define GATEWAY_ACCEPT (&gateway_accept.super.super)
 
 struct command *
-make_gateway_setup(struct command *listen);
-
-extern struct command gateway_setup_command;
-#define GATEWAY_SETUP (&gateway_setup_command.super)
+make_gateway_setup(struct local_info *local);
 
 #endif /* GATEWAY_COMMANDS_H_INCLUDED */
