@@ -2,7 +2,7 @@
  *
  * Simple mark&sweep garbage collector.
  *
- * $Id: gc.h,v 1.11 2001/09/27 12:09:52 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -37,4 +37,7 @@ void gc(void);
 
 void gc_final(void);
 
+struct lsh_object *
+gc_iterate_objects(const struct lsh_class *class, struct lsh_object *o);
+
 #endif /* LSH_GC_H_INCLUDED */
