@@ -79,7 +79,7 @@
 # endif
 #endif /* WITH_ZLIB */
 
-#include "nettle/yarrow.h"
+#include <nettle/yarrow.h>
 
 #include "environ.h"
 #include "format.h"
@@ -749,7 +749,8 @@ spawn_source_process(unsigned *index,
 	
       close (output[0]);
       close (output[1]);
-      
+      close (dev_null);
+
       /* Works also if state->source->arg == NULL */
       execl(state->source->path, state->source->path,
 	    state->source->arg, NULL);
@@ -1218,6 +1219,7 @@ main(int argc, char **argv)
 
   struct yarrow256_ctx yarrow;
   struct yarrow_source sources[NSOURCES];
+  uint8_t seed[YARROW256_SEED_FILE_SIZE];
 
   argp_parse(&main_argp, argc, argv, 0, NULL, options);
 
@@ -1304,7 +1306,8 @@ main(int argc, char **argv)
 	     options->filename);
 
       overwrite = 1;
-      
+
+      /* FIXME: Use O_TRUNC? */
       fd = open(lsh_get_cstring(options->filename),
 		O_WRONLY,
 		0600);
@@ -1356,7 +1359,8 @@ main(int argc, char **argv)
 	      return EXIT_FAILURE;
 	    }
 	}
-      
+
+      /* FIXME: Use O_TRUNC instead? */
       if (ftruncate(fd, 0) < 0)
 	{
 	  werror("Failed to truncate file `%S' %e\n",
@@ -1368,7 +1372,8 @@ main(int argc, char **argv)
 	}
     }
   
-  e = write_raw(fd, sizeof(yarrow.seed_file), yarrow.seed_file);
+  yarrow256_random(&yarrow, sizeof(seed), seed);
+  e = write_raw(fd, sizeof(seed), seed);
 
   if (e)
     {
