@@ -1,6 +1,6 @@
 /* server_keyexchange.c
  *
- * $Id: server_keyexchange.c,v 1.52 2002/03/13 15:48:46 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -21,12 +21,16 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+
 #include "server_keyexchange.h"
 
 #include "atoms.h"
 #include "command.h"
-#include "debug.h"
-#include "dsa.h"
 #include "format.h"
 #include "sexp.h"
 #include "srp.h"
@@ -34,8 +38,6 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-
 #include "server_keyexchange.c.x"
 
 /* GABA:
@@ -78,11 +80,11 @@ do_handle_dh_init(struct packet_handler *c,
     }
   
   /* Send server's message, to complete key exchange */
-  C_WRITE_NOW(connection,
-	      dh_make_server_msg(&closure->dh,
-				 closure->server_key,
-				 closure->hostkey_algorithm,
-				 closure->signer));
+  connection_send_kex(connection,
+		      dh_make_server_msg(&closure->dh,
+					 closure->server_key,
+					 closure->hostkey_algorithm,
+					 closure->signer));
 
   connection->dispatch[SSH_MSG_KEXDH_INIT] = &connection_fail_handler;
 
@@ -128,13 +130,6 @@ do_init_server_dh(struct keyexchange_algorithm *c,
       dh->server_key = lsh_string_dup(key->public);
       dh->signer = key->private;
 
-#if DATAFELLOWS_WORKAROUNDS
-      if ( (hostkey_algorithm == ATOM_SSH_DSS)
-	   && (connection->peer_flags & PEER_SSH_DSS_KLUDGE))
-	{
-	  hostkey_algorithm = ATOM_SSH_DSS_KLUDGE_LOCAL;
-	}
-#endif
       dh->hostkey_algorithm = hostkey_algorithm;
 
       dh->algorithms = algorithms;
@@ -195,7 +190,7 @@ do_srp_server_proof_handler(struct packet_handler *s,
   
   if (response)
     {
-      C_WRITE_NOW(connection, response);
+      connection_send_kex(connection, response);
 
       /* Remember that a user was authenticated. */
       connection->user = self->srp->user;
@@ -261,27 +256,19 @@ do_server_srp_read_verifier(struct abstract_write *s,
   
   else
     {
-      struct sexp *expr = string_to_sexp(SEXP_CANONICAL, file, 1);
-
-      if (!expr)
-	disconnect_kex_failed(self->connection,
-				"S-expression Syntax error in verifier file");
+      self->srp->entry = make_srp_entry(self->srp->user->name, file);
+	  
+      if (!self->srp->entry)
+	disconnect_kex_failed(self->connection, "Invalid SRP verifier");
       else
 	{
-	  self->srp->entry = make_srp_entry(self->srp->user->name, expr);
-	  
-	  if (!self->srp->entry)
-	    disconnect_kex_failed(self->connection, "Invalid SRP verifier");
-	  else
-	    {
-	      /* Success */
+	  /* Success */
 
-	      C_WRITE_NOW(self->connection,
-			  srp_make_reply_msg(&self->srp->dh,
-					     self->srp->entry));
-	      self->connection->dispatch[SSH_MSG_KEXSRP_PROOF]
-		= make_srp_server_proof_handler(self->srp);
-	    }
+	  connection_send_kex(self->connection,
+		      srp_make_reply_msg(&self->srp->dh,
+					 self->srp->entry));
+	  self->connection->dispatch[SSH_MSG_KEXSRP_PROOF]
+	    = make_srp_server_proof_handler(self->srp);
 	}
     }
 }
@@ -347,7 +334,7 @@ make_srp_init_handler(struct srp_server_instance *srp)
      (name srp_server_exchange)
      (super keyexchange_algorithm)
      (vars
-       (dh object dh_method)
+       (dh const object dh_method)
        (db object user_db)))
 */
 
@@ -375,14 +362,11 @@ do_init_server_srp(struct keyexchange_algorithm *s,
 }
 
 struct keyexchange_algorithm *
-make_srp_server(struct dh_method *dh,
+make_srp_server(const struct dh_method *dh,
 		struct user_db *db)
 {
   NEW(srp_server_exchange, self);
 
-  assert(dh->G->add);
-  assert(dh->G->subtract);
-  
   self->super.init = do_init_server_srp;
   self->dh = dh;
   self->db = db;
