@@ -21,19 +21,23 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+#include <errno.h>
+
 #include "server_userauth.h"
 
 #include "charset.h"
 #include "connection.h"
 #include "format.h"
+#include "lsh_string.h"
 #include "ssh.h"
 #include "werror.h"
 #include "xalloc.h"
 
-#include <string.h>
-#include <assert.h>
-#include <errno.h>
-
 
 #define GABA_DEFINE
 #include "server_userauth.h.x"
@@ -88,32 +92,9 @@ format_userauth_success(void)
        (services object alist)))
 */
 
-/* FIXME: This code doesn't handle authentication methods where the
- * result (continuation or exception) is not invoked immediately.
- * There are two problems:
- *
- * 1. Requests are not necessarily replied to in order. That is bad,
- * but can probably be fixed fairly easily the same way that it is
- * done for GLOBAL_REQUEST messages.
- *
- * 2. Packets that are received after a sucessful USERAUTH_REQUEST
- * message, but before it is processed and replied to, must somehow be
- * queued until we know that the user is authenticated for some
- * service to receive them.
- *
- * I think the right thing to do is to serialize userauth requests
- * completely: if a request can't be replied to immediately, put the
- * entire connection on hold until the reply is ready.
- *
- * This code now uses serialization, using connection_lock and
- * connection_unlock. However, the implementation of serialization
- * is rather stupid. And will crash if a userauth method returns to
- * the main loop while the connection is still locked. */
-
-/* FIXME: I don't think the about is true anymore. Locking should
- * work, packets queued up properly while a connection is locked, etc.
- * This is needed for password userauthentication using a password
- * helper process. */
+/* NOTE: Processing of userauth requests is serialized completely: if
+ * a request can't be replied to immediately, put the entire
+ * connection on hold until the reply is ready. */
 
 /* GABA:
    (class
@@ -141,7 +122,7 @@ do_userauth_continuation(struct command_continuation *s,
   /* Unlock connection */
   connection_unlock(self->connection);
   
-  C_WRITE(self->connection, format_userauth_success());
+  connection_send(self->connection, format_userauth_success());
   
   /* Ignore any further userauth messages. */
   for (i = SSH_FIRST_USERAUTH_GENERIC; i < SSH_FIRST_CONNECTION_GENERIC; i++) 
@@ -190,7 +171,7 @@ do_handle_userauth(struct packet_handler *s,
   int requested_service;
   int method;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_USERAUTH_REQUEST)
@@ -290,8 +271,8 @@ do_exc_userauth_handler(struct exception_handler *s,
 	if (self->attempts)
 	  {
 	    self->attempts--;
-	    C_WRITE(self->connection,
-		    format_userauth_failure(self->advertised_methods, 0));
+	    connection_send(self->connection,
+			    format_userauth_failure(self->advertised_methods, 0));
 	  }
 	else
 	  {
@@ -312,7 +293,7 @@ do_exc_userauth_handler(struct exception_handler *s,
 	
 	/* NOTE: We can't NULL e->reply, since the exception is supposed to be constant.
 	 * So we have to dup it, to make the gc happy. */
-	C_WRITE(self->connection, lsh_string_dup(e->reply));
+	connection_send(self->connection, lsh_string_dup(e->reply));
 
 	/* FIXME: Possibly call connection_handle_pending. */	
 	break;
@@ -380,18 +361,18 @@ make_userauth_service(struct int_list *advertised_methods,
   return &self->super;
 }
 
-
-/* None service (which works like external-keyexchange) */
+/* None service, which allows login iff the user was authenticated
+   during keyexchange */
 static void
-do_none_authenticate(struct userauth *s UNUSED,
-                     struct ssh_connection *connection UNUSED,
-                     struct lsh_string *username,
-                     UINT32 service UNUSED,
-                     struct simple_buffer *args,
-                     struct command_continuation *c,
-                     struct exception_handler *e)
+do_none_preauth(struct userauth *s UNUSED,
+		struct ssh_connection *connection UNUSED,
+		struct lsh_string *username,
+		uint32_t service UNUSED,
+		struct simple_buffer *args,
+		struct command_continuation *c,
+		struct exception_handler *e)
 {
-  username = utf8_to_local(username, 1, 1);
+  username = utf8_to_local(username, utf8_paranoid, 1);
   if (!username)
     {
       PROTOCOL_ERROR(e, "Invalid utf8 in username.");
@@ -416,5 +397,48 @@ do_none_authenticate(struct userauth *s UNUSED,
   lsh_string_free(username);
 }
 
-struct userauth server_userauth_none =
-  { STATIC_HEADER, do_none_authenticate };
+struct userauth server_userauth_none_preauth =
+  { STATIC_HEADER, do_none_preauth };
+
+/* GABA:
+   (class
+     (name userauth_none_permit)
+     (super userauth)
+     (vars
+       (user object lsh_user)))
+*/
+
+/* None service, which ignores the username and allows anyone to login. */
+static void
+do_none_permit(struct userauth *s,
+	       struct ssh_connection *connection UNUSED,
+	       struct lsh_string *username,
+	       uint32_t service UNUSED,
+	       struct simple_buffer *args,
+	       struct command_continuation *c,
+	       struct exception_handler *e)
+{
+  CAST(userauth_none_permit, self, s);
+  
+  /* Ignores the username */
+  lsh_string_free(username);
+
+  if (parse_eod(args))
+    {
+      COMMAND_RETURN(c, self->user);
+    }
+  else
+    PROTOCOL_ERROR(e, "Invalid none USERAUTH message.");
+}
+
+struct userauth *
+make_userauth_none_permit(struct lsh_user *user)
+{
+  NEW(userauth_none_permit, self);
+
+  trace("make_userauth_none\n");
+  self->super.authenticate = do_none_permit;
+  self->user = user;
+
+  return &self->super;
+}
