@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: gnu_stuff.h,v 1.3 2002/06/06 20:31:38 _cvs_pont Exp $
+ * @(#) $Id: a978ac24f162a55079cf244b02ef126e92f2ee72 $
  *
  * gnu_stuff.h
  */
@@ -26,10 +26,6 @@
 #ifndef LSFTP_GNU_STUFF_H
 #define LSFTP_GNU_STUFF_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <stdio.h>
 
 void help_option(void);
