@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: misc_fun.h,v 1.2 2002/02/07 12:56:52 nisse Exp $
+ * @(#) $Id: a966bfe26257bc6a6b53944d28d5e884633944b8 $
  *
  * misc_fun.h
  */
@@ -26,10 +26,6 @@
 #ifndef LSFTP_MISC_FUN_H
 #define LSFTP_MISC_FUN_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <unistd.h>
 #include <stdlib.h>
 #include <errno.h>
