@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: str_utils.h,v 1.3 2002/06/06 20:31:38 _cvs_pont Exp $
+ * @(#) $Id: 0772e7471bf61828823abb6dcdfbb65db4f32264 $
  *
  * str_utils.h
  */
@@ -27,10 +27,6 @@
 #ifndef LSFTP_STR_UTILS_H
 #define LSFTP_STR_UTILS_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
