@@ -2,7 +2,7 @@
  *
  * Handle process suspend for the client programs.
  *
- * $Id: suspend.h,v 1.2 2001/09/03 08:31:08 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -34,7 +34,7 @@ suspend_install_handler(void);
 void
 suspend_handle_tty(int fd);
 
-extern struct lsh_callback
+extern struct escape_callback
 suspend_callback;
 
 #endif /* LSH_SUSPEND_H_INCLUDED */
